from bolsonaro.models.omp_forest_classifier import OmpForestBinaryClassifier, OmpForestMulticlassClassifier
from bolsonaro.models.omp_forest_regressor import OmpForestRegressor
from bolsonaro.models.model_parameters import ModelParameters
from bolsonaro.models.similarity_forest_regressor import SimilarityForestRegressor
from bolsonaro.models.kmeans_forest_regressor import KMeansForestRegressor
from bolsonaro.models.ensemble_selection_forest_regressor import EnsembleSelectionForestRegressor
from bolsonaro.data.task import Task

from sklearn.ensemble import RandomForestRegressor, RandomForestClassifier
import os
import pickle


class ModelFactory(object):

    @staticmethod
    def build(task, model_parameters, library=None):
        if task not in [Task.BINARYCLASSIFICATION, Task.REGRESSION, Task.MULTICLASSIFICATION]:
            raise ValueError("Unsupported task '{}'".format(task))

        if task == Task.BINARYCLASSIFICATION:
            if model_parameters.extraction_strategy == 'omp':
                return OmpForestBinaryClassifier(model_parameters)
            elif model_parameters.extraction_strategy == 'random':
                return RandomForestClassifier(**model_parameters.hyperparameters,
                    random_state=model_parameters.seed)
            elif model_parameters.extraction_strategy == 'none':
                return RandomForestClassifier(**model_parameters.hyperparameters,
                    random_state=model_parameters.seed)
            else:
                raise ValueError('Invalid extraction strategy')
        elif task == Task.REGRESSION:
            if model_parameters.extraction_strategy == 'omp':
                return OmpForestRegressor(model_parameters)
            elif model_parameters.extraction_strategy == 'random':
                return RandomForestRegressor(**model_parameters.hyperparameters,
                    random_state=model_parameters.seed)
            elif model_parameters.extraction_strategy == 'similarity':
                return SimilarityForestRegressor(model_parameters)
            elif model_parameters.extraction_strategy == 'kmeans':
                return KMeansForestRegressor(model_parameters)
            elif model_parameters.extraction_strategy == 'ensemble':
                return EnsembleSelectionForestRegressor(model_parameters, library=library)
            elif model_parameters.extraction_strategy == 'none':
                return RandomForestRegressor(**model_parameters.hyperparameters,
                    random_state=model_parameters.seed)
            else:
                raise ValueError('Invalid extraction strategy')
        elif task == Task.MULTICLASSIFICATION:
            if model_parameters.extraction_strategy == 'omp':
                return OmpForestMulticlassClassifier(model_parameters)
            elif model_parameters.extraction_strategy == 'random':
                return RandomForestClassifier(**model_parameters.hyperparameters,
                    random_state=model_parameters.seed)
            elif model_parameters.extraction_strategy == 'none':
                return RandomForestClassifier(**model_parameters.hyperparameters,
                    random_state=model_parameters.seed)
            else:
                raise ValueError('Invalid extraction strategy')
